%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               PREPROCESS                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex mexTPA.cpp -largeArrayDims;

[B, n] = Import('Slashdot.txt');
c = 0.15;
epsilon = 0.000000001;
S = 5; T = 15; merging = (1-(1-c)^T);
sampleNum = 30;

maxNumCompThreads(1);
fprintf('\n###########################################################\n');
fprintf('PREPROCESS PHASE STARTS\n');

sd = zeros(1,1);
sd(1,1) = 0;
cc = zeros(1,1);
cc(1,1) = c;
iterS = zeros(1,1);
iterS(1,1) = S;
iterT = zeros(1,1);
itert(1,1) = T;
[R] = mexTPA(B, sd, cc, iterS, iterT);
r_global = R(1:n);
elapsedTime = R(n+1);
fprintf('[TPA] Stranger Approximation: elapsed time is %g seconds\n', elapsedTime);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                ONLINE                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('\n###########################################################\n');
fprintf('ONLINE PHASE STARTS\n');
for sample = 1 : sampleNum 
    seed = randi(n);
    
    sd(1,1) = seed;
    [R] = mexTPA(B, sd, cc, iterS, iterT);
    r_local = R(1:n);
    time_local = R(n+1);
    dead_end = R(n+2);
    
    start = clock; 
    if dead_end == 0
        r_tpa = merging * r_local + (1-merging) * r_global; 
    else
        r_tpa = r_local;
    end
    time = etime(clock, start);
    
    elapsedTime = time_local + time;
    fprintf('[TPA] Neighbor Approximation: elapsed time is %g seconds\n', elapsedTime);
end
